#include "defo.h"
#include "MyIntervalFunc.h"

MyInterval sqrt2;   // = sqrt(MyInterval(2.0));
MyInterval sqrt3;   // = sqrt(MyInterval(3.0));
MyInterval piConst; // = interval::pi();

void initIntervalGlobals()
{
    sqrt2 = sqrt(MyInterval(2.0));
    sqrt3 = sqrt(MyInterval(3.0));
    piConst = MyInterval::pi();
}

bool emptyIntersection(const MyInterval &X,const   MyInterval &Y)
// returns: True iff X \cap Y=\emptyset
{
  return (X.rightBound() < Y.leftBound() || Y.rightBound() < X.leftBound());
}


MyDouble maksimum(const MyDouble &x,const  MyDouble &y)
{
  return (x >= y) ? x : y;
}

MyDouble minimum(const MyDouble &x,const  MyDouble &y)
{
  return (x <= y) ? x : y;
}

bool myIntersection( MyInterval X,  MyInterval Y, MyInterval &result)
// returns: True iff X \cap Y \neq \empty; in such case on output result=X \cap Y 
//          otherwise result is not changed
{
  if (emptyIntersection(X, Y))
    return false;

  result = MyInterval(maksimum(X.leftBound(), Y.leftBound()), minimum(X.rightBound(), Y.rightBound()));
  return true;
}

MyDouble diam(const MyInterval &x)
{
    MyInterval di=MyInterval(x.rightBound()) - x.leftBound();
	return di.rightBound();
}


bool subSetInterior(const MyInterval &X,const MyInterval &Y)
// returns X \subset Interior of Y
{
  return ((X.leftBound() > Y.leftBound()) &&
          ((X.rightBound() < Y.rightBound())));
}

bool subSet(const MyInterval &X,const MyInterval &Y)
// returns X \subset  Y
{
  return ((X.leftBound() >= Y.leftBound()) &&
          ((X.rightBound() <= Y.rightBound())));
}


bool isomorphismTest(const MyMatrix &DFF)
// if returns true then DFF is isomorphism
//  when false - then DFF did not pass our test for being the isomorphism
{
  try
  {
    MyMatrix invDF = capd::matrixAlgorithms::gaussInverseMatrix(DFF);
  }
  catch (std::runtime_error)
  {
    return false;
  }
  return true; 
}


